.AvatarCard_components__heTwn .AvatarCard_semi-icon-close__9hwya{visibility:hidden;color:var(--semi-color-danger)}.AvatarCard_components__heTwn:hover .AvatarCard_semi-icon-close__9hwya{visibility:visible}.AvatarCard_components__heTwn{height:52px;box-sizing:border-box;display:flex;align-items:center;justify-content:space-between;padding:10px 12px}.AvatarCard_components__heTwn:hover{background-color:var(--semi-color-fill-0)}.AvatarCard_components__heTwn .AvatarCard_info__Gzwh8{margin-left:8px;flex-grow:1}.AvatarCard_components__heTwn .AvatarCard_name__mOADG{font-size:14px;line-height:20px}.AvatarCard_components__heTwn .AvatarCard_email__ncXyR{font-size:12px;line-height:16px;color:var(--semi-color-text-2)}