(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[357],{1211:function(t,r,n){Promise.resolve().then(n.bind(n,62161))},30695:function(t,r,n){"use strict";n.d(r,{_i:function(){return a},gz:function(){return c},hw:function(){return s},sj:function(){return i},wG:function(){return o}});var e=n(20357);async function o(t,r){var n;let{arg:o}=r,a=await fetch((null!==(n=e.env.NEXT_PUBLIC_API_SERVER)&&void 0!==n?n:"")+t,{method:"POST",headers:{"Content-Type":"application/json"},body:JSON.stringify(o)});if(!(a.status>=200&&a.status<300))throw Error(await a.text());let i=await a.json();if(!a.ok)throw Error(i.message);return i}let a=async function(){for(var t,r=arguments.length,n=Array(r),o=0;o<r;o++)n[o]=arguments[o];let a=await fetch((null!==(t=e.env.NEXT_PUBLIC_API_SERVER)&&void 0!==t?t:"")+n[0],n[1]);if(!a.ok)throw Error(await a.text());return a.json()},i=async(t,r)=>{var n;let o=await fetch((null!==(n=e.env.NEXT_PUBLIC_API_SERVER)&&void 0!==n?n:"")+t,r);if(!o.ok)throw Error(await o.text());return o};async function s(t,r){var n;let{arg:o}=r,a=await fetch("".concat(null!==(n=e.env.NEXT_PUBLIC_API_SERVER)&&void 0!==n?n:"").concat(t,"/").concat(o),{method:"DELETE"});if(!a.ok)throw Error(await a.text());return a}async function c(t,r){var n;let{arg:o}=r,a=await fetch("".concat(null!==(n=e.env.NEXT_PUBLIC_API_SERVER)&&void 0!==n?n:"").concat(t),{method:"PUT",headers:{"Content-Type":"application/json"},body:JSON.stringify(o)});if(!a.ok)throw Error(await a.text());return a}},62161:function(t,r,n){"use strict";n.r(r),n.d(r,{default:function(){return l}});var e=n(57437),o=n(48936),a=n(91617),i=n(30695),s=n(42132),c=n(94427);function l(){let{Header:t,Footer:r,Sider:n,Content:l}=o.Layout,{data:u,error:d,isLoading:f}=(0,a.ZP)("/v1/status",i._i);return f?(0,e.jsx)(o.yC,{size:"large"}):(0,e.jsxs)(e.Fragment,{children:[(0,e.jsx)(t,{style:{backgroundColor:"var(--semi-color-bg-1)"},children:(0,e.jsx)(o.JL,{style:{border:"none"},header:(0,e.jsxs)(e.Fragment,{children:[(0,e.jsx)("div",{style:{backgroundColor:"rgba(var(--semi-lime-2), 1)",borderRadius:"var(--semi-border-radius-large)",color:"var(--semi-color-bg-0)",display:"flex",padding:"6px"},children:(0,e.jsx)(s.Z,{size:"large"})}),(0,e.jsx)("h4",{style:{marginLeft:"12px"},children:"任务平台"})]}),mode:"horizontal"})}),(0,e.jsx)(l,{style:{paddingLeft:12,paddingRight:12,backgroundColor:"var(--semi-color-bg-0)"},children:(0,e.jsx)("main",{children:(0,e.jsx)(c.L,{data:u})})})]})}}},function(t){t.O(0,[798,674,86,949,753,293,406,862,936,617,361,971,23,744],function(){return t(t.s=1211)}),_N_E=t.O()}]);