(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[931],{27273:function(e,i,t){Promise.resolve().then(t.bind(t,76774))},76774:function(e,i,t){"use strict";t.r(i);var n=t(57437);i.default=()=>(0,n.jsx)("iframe",{style:{borderWidth:0},height:"100%",src:"https://biliup.github.io/biliup/docs/guide/changelog/"})}},function(e){e.O(0,[971,23,744],function(){return e(e.s=27273)}),_N_E=e.O()}]);